package com.engine.organization.wrapper;


import com.alibaba.fastjson.JSON;
import com.engine.common.util.ServiceUtil;
import com.engine.organization.annotation.Log;
import com.engine.organization.entity.scheme.param.GradeSearchParam;
import com.engine.organization.enums.LogModuleNameEnum;
import com.engine.organization.enums.OperateTypeEnum;
import com.engine.organization.service.GradeService;
import com.engine.organization.service.impl.GradeServiceImpl;
import com.engine.organization.util.OrganizationWrapper;
import org.apache.ibatis.annotations.Param;
import weaver.hrm.User;

import java.util.Collection;
import java.util.Map;

/**
 * @description:
 * @author:dxfeng
 * @createTime: 2022/05/11
 * @version: 1.0
 */
public class GradeWrapper extends OrganizationWrapper {


    private GradeService getGradeService(User user) {
        return ServiceUtil.getService(GradeServiceImpl.class, user);
    }


    /**
     * 职级列表
     *
     * @param params
     * @return
     */
    public Map<String, Object> listPage(Map<String, Object> params) {
        return getGradeService(user).listPage(params);
    }

    /**
     * 新增职级
     *
     * @param param
     * @return
     */
    @Log(operateType = OperateTypeEnum.ADD, operateModule = LogModuleNameEnum.GRADE, operateDesc = "新增职等")
    public int saveGrade(GradeSearchParam param) {
        int saveGrade = getGradeService(user).saveGrade(param);
        writeOperateLog(new Object() {
        }.getClass(), "新增职等【" + JSON.toJSONString(param) + "】");
        return saveGrade;
    }

    /**
     * 更新职级
     *
     * @param param
     * @return
     */
    @Log(operateType = OperateTypeEnum.UPDATE, operateModule = LogModuleNameEnum.GRADE, operateDesc = "更新职级")
    public int updateGrade(GradeSearchParam param) {
        int updateGrade = getGradeService(user).updateGrade(param);
        writeOperateLog(new Object() {
        }.getClass(), "更新职级【" + JSON.toJSONString(param) + "】");
        return updateGrade;
    }

    /**
     * 更新禁用标记
     *
     * @param params
     */
    @Log(operateType = OperateTypeEnum.UPDATE, operateModule = LogModuleNameEnum.GRADE, operateDesc = "更新职级禁用标识")
    public int updateForbiddenTagById(GradeSearchParam params) {
        int updateForbiddenTagById = getGradeService(user).updateForbiddenTagById(params);
        writeOperateLog(new Object() {
        }.getClass(), "更新职级禁用标识【" + JSON.toJSONString(params) + "】");
        return updateForbiddenTagById;
    }

    /**
     * 根据ID批量删除
     *
     * @param ids
     */
    @Log(operateType = OperateTypeEnum.DELETE, operateModule = LogModuleNameEnum.GRADE, operateDesc = "删除职等")
    public int deleteByIds(@Param("ids") Collection<Long> ids) {
        int deleteByIds = getGradeService(user).deleteByIds(ids);
        writeOperateLog(new Object() {
        }.getClass(), "删除职等【" + JSON.toJSONString(ids) + "】");
        return deleteByIds;
    }

    /**
     * 获取搜索条件
     *
     * @param params
     * @return
     */
    public Map<String, Object> getSearchCondition(Map<String, Object> params) {
        return getGradeService(user).getSearchCondition(params);
    }


    /**
     * 获取新增表单
     *
     * @param params
     * @return
     */
    public Map<String, Object> getGradeForm(Map<String, Object> params) {
        return getGradeService(user).getGradeForm(params);
    }

    /**
     * 获取列表页面按钮信息
     *
     * @return
     */
    public Map<String, Object> getHasRight() {
        return getGradeService(user).getHasRight();
    }

    /**
     * 获取列表tabs
     *
     * @return
     */
    public Map<String, Object> getTabInfo() {
        return getGradeService(user).getTabInfo();
    }
}
