package com.engine.organization.wrapper;

import com.alibaba.fastjson.JSON;
import com.engine.common.util.ServiceUtil;
import com.engine.organization.annotation.Log;
import com.engine.organization.entity.hrmresource.param.HrmResourceSearchParam;
import com.engine.organization.entity.searchtree.SearchTreeParams;
import com.engine.organization.enums.LogModuleNameEnum;
import com.engine.organization.enums.OperateTypeEnum;
import com.engine.organization.service.HrmResourceService;
import com.engine.organization.service.impl.HrmResourceServiceImpl;
import com.engine.organization.util.OrganizationWrapper;
import weaver.hrm.User;

import java.util.Map;

/**
 * @author:dxfeng
 * @createTime: 2022/06/21
 * @version: 1.0
 */
public class HrmResourceWrapper extends OrganizationWrapper {
    private HrmResourceService getHrmResourceService(User user) {
        return ServiceUtil.getService(HrmResourceServiceImpl.class, user);
    }


    public Map<String, Object> getSearchTree(SearchTreeParams params) {
        return getHrmResourceService(user).getSearchTree(params);
    }

    public Map<String, Object> listPage(HrmResourceSearchParam params) {
        return getHrmResourceService(user).listPage(params);
    }

    public Map<String, Object> getSaveForm() {
        return getHrmResourceService(user).getSaveForm();
    }

    @Log(operateType = OperateTypeEnum.ADD, operateModule = LogModuleNameEnum.RESOURCE, operateDesc = "新增人员")
    public Long saveBaseForm(Map<String, Object> params) {
        Long resourceId = getHrmResourceService(user).saveBaseForm(params);
        writeOperateLog(new Object() {
        }.getClass(), "新增人员【" + JSON.toJSONString(params) + "】");
        return resourceId;
    }

    public Map<String, Object> getBaseForm(Map<String, Object> params) {
        return getHrmResourceService(user).getBaseForm(params);
    }

    @Log(operateType = OperateTypeEnum.UPDATE, operateModule = LogModuleNameEnum.RESOURCE, operateDesc = "更新人员信息")
    public int updateForm(Map<String, Object> params) {
        int updateForm = getHrmResourceService(user).updateForm(params);
        writeOperateLog(new Object() {
        }.getClass(), "更新人员信息【" + JSON.toJSONString(params) + "】");
        return updateForm;
    }

    public Map<String, Object> getSearchCondition(Map<String, Object> params) {
        return getHrmResourceService(user).getSearchCondition(params);
    }

    public Map<String, Object> getHasRight() {
        return getHrmResourceService(user).getHasRight();
    }


}
