package com.engine.organization.wrapper;

import com.alibaba.fastjson.JSON;
import com.engine.common.util.ServiceUtil;
import com.engine.organization.annotation.Log;
import com.engine.organization.entity.TreeData;
import com.engine.organization.entity.scheme.param.LevelSearchParam;
import com.engine.organization.enums.LogModuleNameEnum;
import com.engine.organization.enums.OperateTypeEnum;
import com.engine.organization.service.LevelService;
import com.engine.organization.service.impl.LevelServiceImpl;
import com.engine.organization.util.OrganizationWrapper;
import org.apache.ibatis.annotations.Param;
import weaver.hrm.User;

import java.util.Collection;
import java.util.Map;

/**
 * @description:
 * @author:dxfeng
 * @createTime: 2022/05/10
 * @version: 1.0
 */
public class LevelWrapper extends OrganizationWrapper {
    private LevelService getLevelService(User user) {
        return ServiceUtil.getService(LevelServiceImpl.class, user);
    }

    /**
     * 职等列表
     *
     * @param params
     * @return
     */
    public Map<String, Object> listPage(Map<String, Object> params) {
        return getLevelService(user).listPage(params);
    }

    /**
     * 新增职等
     *
     * @param param
     * @return
     */
    @Log(operateType = OperateTypeEnum.ADD, operateModule = LogModuleNameEnum.LEVEL, operateDesc = "新增职等")
    public int saveLevel(LevelSearchParam param) {
        int saveLevel = getLevelService(user).saveLevel(param);
        writeOperateLog(new Object() {
        }.getClass(), "新增职等【" + JSON.toJSONString(param) + "】");
        return saveLevel;
    }

    /**
     * 更新职等
     *
     * @param param
     * @return
     */
    @Log(operateType = OperateTypeEnum.UPDATE, operateModule = LogModuleNameEnum.LEVEL, operateDesc = "更新职等")
    public int updateLevel(LevelSearchParam param) {
        int updateLevel = getLevelService(user).updateLevel(param);
        writeOperateLog(new Object() {
        }.getClass(), "更新职等【" + JSON.toJSONString(param) + "】");
        return updateLevel;
    }

    /**
     * 更新禁用标记
     *
     * @param params
     */
    @Log(operateType = OperateTypeEnum.UPDATE, operateModule = LogModuleNameEnum.LEVEL, operateDesc = "更新职等禁用标识")
    public int updateForbiddenTagById(LevelSearchParam params) {
        int updateForbiddenTagById = getLevelService(user).updateForbiddenTagById(params);
        writeOperateLog(new Object() {
        }.getClass(), "更新职等禁用标识【" + JSON.toJSONString(params) + "】");
        return updateForbiddenTagById;
    }

    /**
     * 根据ID批量删除
     *
     * @param ids
     */
    @Log(operateType = OperateTypeEnum.DELETE, operateModule = LogModuleNameEnum.LEVEL, operateDesc = "删除职等")
    public int deleteByIds(@Param("ids") Collection<Long> ids) {
        int deleteByIds = getLevelService(user).deleteByIds(ids);
        writeOperateLog(new Object() {
        }.getClass(), "删除职等【" + JSON.toJSONString(ids) + "】");
        return deleteByIds;
    }

    /**
     * 获取搜索条件
     *
     * @param params
     * @return
     */
    public Map<String, Object> getSearchCondition(Map<String, Object> params) {
        return getLevelService(user).getSearchCondition(params);
    }


    /**
     * 获取新增表单
     *
     * @param params
     * @return
     */
    public Map<String, Object> getLevelForm(Map<String, Object> params) {
        return getLevelService(user).getLevelForm(params);
    }

    /**
     * 获取列表页面按钮信息
     *
     * @return
     */
    public Map<String, Object> getTableBtn() {
        return getLevelService(user).getTableBtn();
    }

    /**
     * 获取列表tabs
     *
     * @return
     */
    public Map<String, Object> getTabInfo() {
        return getLevelService(user).getTabInfo();
    }

    public TreeData getTreeData() {
        return getLevelService(user).getTreeData();
    }
}
