package com.engine.organization.wrapper;

import com.alibaba.fastjson.JSON;
import com.engine.common.util.ServiceUtil;
import com.engine.organization.annotation.Log;
import com.engine.organization.entity.fieldset.vo.TypeTreeVO;
import com.engine.organization.entity.scheme.param.SchemeSearchParam;
import com.engine.organization.enums.LogModuleNameEnum;
import com.engine.organization.enums.OperateTypeEnum;
import com.engine.organization.service.SchemeService;
import com.engine.organization.service.impl.SchemeServiceImpl;
import com.engine.organization.util.OrganizationWrapper;
import org.apache.ibatis.annotations.Param;
import weaver.hrm.User;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @Author dxfeng
 * @description:
 * @Date 2022/5/9
 * @Version V1.0
 **/
public class SchemeWrapper extends OrganizationWrapper {

    private SchemeService getSchemeService(User user) {
        return ServiceUtil.getService(SchemeServiceImpl.class, user);
    }

    /**
     * 等级方案列表
     *
     * @param params
     * @return
     */
    public Map<String, Object> listPage(Map<String, Object> params) {
        return getSchemeService(user).listPage(params);
    }

    /**
     * 新增等级方案
     *
     * @param param
     * @return
     */
    @Log(operateType = OperateTypeEnum.ADD, operateModule = LogModuleNameEnum.SCHEME, operateDesc = "新增等级方案")
    public int save(SchemeSearchParam param) {
        int save = getSchemeService(user).save(param);
        writeOperateLog(new Object() {
        }.getClass(), "新增等级方案【" + JSON.toJSONString(param) + "】");
        return save;
    }

    /**
     * 更新等级方案信息
     *
     * @param param
     * @return
     */
    @Log(operateType = OperateTypeEnum.UPDATE, operateModule = LogModuleNameEnum.SCHEME, operateDesc = "更新等级方案")
    public int updateScheme(SchemeSearchParam param) {
        int updateScheme = getSchemeService(user).updateScheme(param);
        writeOperateLog(new Object() {
        }.getClass(), "更新等级方案【" + JSON.toJSONString(param) + "】");
        return updateScheme;
    }

    /**
     * 更新禁用标记
     *
     * @param params
     */
    @Log(operateType = OperateTypeEnum.UPDATE, operateModule = LogModuleNameEnum.SCHEME, operateDesc = "更新等级方案禁用标识")
    public int updateForbiddenTagById(SchemeSearchParam params) {
        int updateForbiddenTagById = getSchemeService(user).updateForbiddenTagById(params);
        writeOperateLog(new Object() {
        }.getClass(), "更新等级方案禁用标识【" + JSON.toJSONString(params) + "】");
        return updateForbiddenTagById;
    }

    /**
     * 根据ID批量删除等级方案信息
     *
     * @param ids
     */
    @Log(operateType = OperateTypeEnum.DELETE, operateModule = LogModuleNameEnum.SCHEME, operateDesc = "删除等级方案")
    public int deleteByIds(@Param("ids") Collection<Long> ids) {
        int deleteByIds = getSchemeService(user).deleteByIds(ids);
        writeOperateLog(new Object() {
        }.getClass(), "更新等级方案禁用标识【" + JSON.toJSONString(ids) + "】");
        return deleteByIds;
    }


    /**
     * 获取搜索条件
     *
     * @param params
     * @return
     */
    public Map<String, Object> getSearchCondition(Map<String, Object> params) {
        return getSchemeService(user).getSearchCondition(params);
    }


    /**
     * 获取新增表单
     *
     * @param params
     * @return
     */
    public Map<String, Object> getSchemeForm(Map<String, Object> params) {
        return getSchemeService(user).getSchemeForm(params);
    }

    /**
     * 获取列表页面按钮信息
     *
     * @return
     */
    public Map<String, Object> getTableBtn() {
        return getSchemeService(user).getTableBtn();
    }

    public List<TypeTreeVO> getTreeData() {
        return getSchemeService(user).getTreeData();
    }
}
