package com.engine.organization.wrapper;

import com.alibaba.fastjson.JSON;
import com.engine.common.util.ServiceUtil;
import com.engine.organization.annotation.Log;
import com.engine.organization.entity.staff.param.StaffSearchParam;
import com.engine.organization.enums.LogModuleNameEnum;
import com.engine.organization.enums.OperateTypeEnum;
import com.engine.organization.service.StaffService;
import com.engine.organization.service.impl.StaffServiceImpl;
import com.engine.organization.util.OrganizationWrapper;
import weaver.hrm.User;

import java.util.Collection;
import java.util.Map;

/**
 * @description:
 * @author:dxfeng
 * @createTime: 2022/05/25
 * @version: 1.0
 */
public class StaffWrapper extends OrganizationWrapper {
    private StaffService getStaffService(User user) {
        return ServiceUtil.getService(StaffServiceImpl.class, user);
    }

    /**
     * 编制列表
     *
     * @param params
     * @return
     */
    public Map<String, Object> listPage(StaffSearchParam params) {
        return getStaffService(user).listPage(params);
    }

    /**
     * 新增编制
     *
     * @param param
     * @return
     */
    @Log(operateType = OperateTypeEnum.ADD, operateModule = LogModuleNameEnum.STAFF, operateDesc = "新增编制上报")
    public int saveStaff(StaffSearchParam param) {
        int saveStaff = getStaffService(user).saveStaff(param);
        writeOperateLog(new Object() {
        }.getClass(), "新增编制上报【" + JSON.toJSONString(param) + "】");
        return saveStaff;
    }

    /**
     * 更新编制
     *
     * @param param
     * @return
     */
    @Log(operateType = OperateTypeEnum.UPDATE, operateModule = LogModuleNameEnum.STAFF, operateDesc = "更新编制上报")
    public int updateStaff(StaffSearchParam param) {
        int updateStaff = getStaffService(user).updateStaff(param);
        writeOperateLog(new Object() {
        }.getClass(), "更新编制上报【" + JSON.toJSONString(param) + "】");
        return updateStaff;
    }


    /**
     * 根据ID批量删除
     *
     * @param ids
     */
    @Log(operateType = OperateTypeEnum.DELETE, operateModule = LogModuleNameEnum.STAFF, operateDesc = "删除编制上报")
    public int deleteByIds(Collection<Long> ids) {
        int deleteByIds = getStaffService(user).deleteByIds(ids);
        writeOperateLog(new Object() {
        }.getClass(), "删除编制上报【" + JSON.toJSONString(ids) + "】");
        return deleteByIds;
    }

    /**
     * 获取搜索条件
     *
     * @param params
     * @return
     */
    public Map<String, Object> getSearchCondition(Map<String, Object> params) {
        return getStaffService(user).getSearchCondition(params);
    }


    /**
     * 获取新增表单
     *
     * @param params
     * @return
     */
    public Map<String, Object> getForm(Map<String, Object> params) {
        return getStaffService(user).getForm(params);
    }

    /**
     * 获取列表页面按钮信息
     *
     * @return
     */
    public Map<String, Object> getHasRight() {
        return getStaffService(user).getHasRight();
    }
}
